@@ -158,6 +158,7 @@ hb_font_get_glyph_v_advance_nil (hb_font_t *font HB_UNUSED,
 				 hb_codepoint_t glyph,
 				 void *user_data HB_UNUSED)
 {
+  /* TODO use font_extents.ascender+descender */
   return font->y_scale;
 }
 static hb_position_t
@@ -1538,6 +1539,8 @@ hb_font_get_ppem (hb_font_t *font,
 }
 
 
+#ifndef HB_DISABLE_DEPRECATED
+
 /*
  * Deprecated get_glyph_func():
  */
@@ -1660,3 +1663,5 @@ hb_font_funcs_set_glyph_func (hb_font_funcs_t *ffuncs,
 					  trampoline,
 					  trampoline_destroy);
 }
+
+#endif /* HB_DISABLE_DEPRECATED */
